/* PCIe-DIO-24HS VxWorks 7 (0600) "driver" Acceptance Test Program */

/* Copyright (c) 2019, ACCES I/O Products, Inc.
 * 10623 Roselle St, San Diego CA, 92121-1506 USA
 * +1-858-550-9559 | +1-800-326-1649 | sales@accesio.com
 * 
 * The right to copy, distribute, modify, or otherwise make use of this software 
 * may be pursuant to the terms of an applicable ACCES I/O Products, Inc. 
 * (ACCES) license agreement, or in the absence of such agreement, is limited to 
 * use with ACCES hardware products.
 */

/*
modification history
--------------------
13feb19,j2h  Initial release.
*/

/*
DESCRIPTION

Test program for ACCES I/O Products DIO cards driver.

*/

#include <vxWorks.h>
#include <private/iosLibP.h>
#include <hwif/buslib/pciDefines.h>
#include <stdio.h>
#include <string.h>
#include <usrLib.h>                     /* for devs() */
#include <hwif/vxBus.h>
#include <hwif/vxbus/vxbBusType.h>
#include <subsys/int/vxbIntLib.h>
//#include <hwif/buslib/vxbPciLib.h>

#include "vxbAIO_IDIO.h"

IMPORT void sysMsDelay (int);   /* delays int msecs */

#define NO_ARG_CALL_TEST(FNAME)                                     \
do { if (FNAME () == OK)                                            \
        printf (#FNAME "() OK.\n");                                 \
    else                                                            \
        {                                                           \
        printf (#FNAME "() failed with errno = %d.\n", errno);      \
        goto L_err;                                                 \
        }                                                           \
    }                                                               \
while (0)

int accesDioTestISRcount;

/******************************************************************************
*
* accesDioTestISR - simple ISR for interrupt test--just increment a count
*/

STATUS accesDioTestISR (void)
    {
    accesDioTestISRcount++;
    return OK;
    }

/******************************************************************************
*
* accesDioIntTest - test DIO card Change-of-State interrupts
* 
* Do all steps to enable interrupts, wait 30 seconds for the user to short a
* pin, and report # interrupts received. A quick paper-clip short will generate
* a few dozen interrupts. See code for details.
*/

STATUS accesDioIntTest (void)
    {
    /***** TEST: Interrupts. */ 

	accesDioTestISRcount = 0; 
	
    /* Establish our ISR callback function and enable interrupts for 30 seconds. */
    NO_ARG_CALL_TEST(accesDioOpen);
    
    if (accesDioIntCallbackInstall (accesDioTestISR) == ERROR)
        { printf ("accesDioIntTest:%d: failed with errno = %d.\n", __LINE__, errno); goto L_err; }

    /* Remember! Ports must be set for input for interrupts to work. */
    if (accesDioConfigure(PORT_DIR_INPUT) == OK) 
        printf ("accesDioConfigure() OK\n");
    else
        { printf ("accesDioTest:%d: failed with errno = %d.\n", __LINE__, errno); goto L_err; }

    accesDioIntClear();
    
    if (accesDioIntEnable () == ERROR)
        { printf ("accesDioIntTest:%d: failed with errno = %d.\n", __LINE__, errno); goto L_err; }

    accesDioIntClear();
    
    printf ("Connect a digital transition source to any digital input and cause a known\n"
    		"number of edges to be generated\n"
            "Our callback function, accesDioTestISR(), increments a local counter which\n"
            "is reported at the end of the test.\n"
    		"T13 Verify reported count matches generated number of edges.\n");
    
      
    sysMsDelay (10000);
    
    NO_ARG_CALL_TEST(accesDioClose);
    
    printf ("Interrupt test done. accesDioTestISRcount = %d\n", accesDioTestISRcount);

    return OK;
    
    /* In Workbench, place a breakpoint here to break on errors. */
L_err:
    printf ("ERROR: At L_err label. errno if relevant = %d.\n", errno);
    return ERROR;
    }

/******************************************************************************
*
* AIO_AcceptanceTest - test DIO driver functions
* 
* Main test for DIO card. Flash LEDs in various ways. See code for details.
*/

STATUS AIO_AcceptanceTest (void)
    {
    UINT8  controlWord, TTLIn, ISOIn, FETOut;
    int    cycle;
    UINT32 pinCnt;
    UINT8 ports;    
    
    printf("PRELIMINARY / ALPHA");

    /* T 0 */ /* Verify accesDioShow */
    printf("Please confirm the output of accesDioShow indicates installed");
    accesDioShow();
	printf ("T 0 Verify onscreen data (BARs above are non-zero)...10 seconds\n");
	sysMsDelay(10000);

    
    
    /* T 1 */ /* Open the card */
    if (accesDioOpen() == OK)
        printf ("T 1 Pass: accesDioOpen() OK\n");
    else
        { printf ("T 1 Fail accesDioOpen() failed @ line %d with errno = %d.\n", __LINE__, errno); goto L_err; }    	
    
    
    
    /* T 2 */ /* Configure TTL as all outputs */
    if (accesDioConfigure( PORT_DIR_OUTPUT) == OK)
        printf ("T 2 Pass: accesDioConfigure() OK\n");
    else
        { printf ("T 2 Fail @ line %d with errno = %d.\n", __LINE__, errno); goto L_err; }
    
    
    
    /* T 3 */ /* Readback control word */
    accesDioReadControl (&controlWord);
    printf ("T 3    accesDioReadControl read = 0x%02x\n"
    		"T 3    Expected 0x80\n"
    		"%s\n", 
    		controlWord, 
			controlWord==0x03 ? "T 3 Pass: control word read" : "T 3 Fail" );

    
    
    /* T 4 */ /* Readback data */
	accesIdioWriteTTL(0xAA);
	accesDioReadPorts(NULL, &TTLIn);
    printf ("T 4    TTL Data Read = 0x%08hhx\n"
    		"T 4    Expected 0xAA\n"
    		"%s\n", 
			TTLIn, TTLIn==0xAA ? "T 4 Pass: data readback" : "T 4 Fail" );
	
	

    
    
    
    
    
    
    
    /* T 6 */ /* Speed test */
    printf("T 6 Verify on oscope that any / all TTL bits toggle not slower than 5ms/edge\n");
    for (cycle = 1; cycle <= 1000000; cycle++)
    {
    	RelOutPortB(0, 0, 0xFF);
    	RelOutPortB(0, 0, 0x00);
    }
    

    
    /* T 7 */ /* Functional */
    printf("T 7 Verify LED display shows 1 walking bit on all TTL bits, twice\n");
    for (cycle = 1; cycle <= 2; cycle++)
        {
        ports = 1;
        for (pinCnt = 1; pinCnt <= 8; pinCnt++)
            {
            accesIdioWriteTTL (ports);
            ports = (UINT8)(ports << 1);
            sysMsDelay (100);
            }
        
        for (pinCnt = 8; pinCnt >= 1; pinCnt--)
            {
            accesIdioWriteTTL (ports);
            ports = ports >> 1;
            sysMsDelay (100);
            }
        }
  

    
    
    if (accesDioClose() == OK)
    	printf ("T11 Pass: accesDioClose() OK\n");
    else
        { printf ("T11 Fail: accesDioClose() failed @ line %d with errno = %d.\n", __LINE__, errno); goto L_err; }    	
  
    
    /* T13 */ /* IRQ stuff */
    accesDioIntTest(); 
    
    printf ("AIO_AcceptanceTest() Done.\n");
    return OK;
    
    /* In Workbench, place a breakpoint here to break on errors. */
L_err:
    printf ("ERROR: At L_err label. errno if relevant = %d.\n", errno);
    return ERROR;
    }


